@@ -5,6 +5,12 @@
  *  for the inverse gaussian distribution. See ../R/InverseGaussian.R
  *  for details.
  *
+ *  We work with the density expressed as
+ *
+ *    (2 pi phi x^3)^(-1/2) exp(- u^2/(2 phi x))
+ *
+ *  with u = (x - mu)/mu.
+ *
  *  The code for functions [dpqr]invgauss() is a C implementation of
  *  functions of the same functions in package statmod; see:
  *
@@ -24,40 +30,36 @@
 
 double dinvgauss(double x, double mu, double phi, int give_log)
 {
-    /*  We work with the density expressed as
-     *
-     *  (2 pi phi x^3)^(-1/2) exp(- u^2/(2 phi x))
-     *
-     *  with u = (x - mu)/mu. The limiting cases are:
-     *
-     *  1. phi = Inf: spike at zero;
-     *  2. mu = Inf: inverse chi-square distribution [a.k.a inverse gamma
-     *               with shape = 1/2, scale = 1/(2 * phi)].
-     */
-
 #ifdef IEEE_754
     if (ISNAN(x) || ISNAN(mu) || ISNAN(phi))
 	return x + mu + phi;
 #endif
-    if (mu <= 0.0 || phi <= 0.0)
+    if (mu <= 0.0)
         return R_NaN;
+    if (phi <= 0)
+    {
+	if (phi < 0) return R_NaN;
+	/* phi == 0 */
+	return (x == 0.0) ? R_PosInf : ACT_D__0;
+    }
 
     if (!R_FINITE(x) || x < 0.0)
 	return ACT_D__0;
 
-    /* limiting case phi = Inf */
+    /* limiting case phi = Inf: spike at zero */
     if (x == 0)
 	return R_FINITE(phi) ? ACT_D__0 : R_PosInf;
 
-    /* limiting case mu = Inf */
+    /* limiting case mu = Inf: inverse chi-square distribution [a.k.a
+     * inverse gamma with shape = 1/2, scale = 1/(2 * phi)] */
     if (!R_FINITE(mu))
 	return ACT_D_exp(-(log(phi) + 3 * log(x) + 1/phi/x)/2 - M_LN_SQRT_2PI);
 
     /* standard cases */
-    double xm = x/mu;
-    double phim = phi * mu;
+    x = x/mu;
+    phi = phi * mu;
 
-    return ACT_D_exp(-(log(phim) + 3 * log(xm) + R_pow_di(xm - 1, 2)/phim/xm)/2
+    return ACT_D_exp(-(log(phi) + 3 * log(x) + R_pow_di(x - 1, 2)/phi/x)/2
 		     - M_LN_SQRT_2PI - log(mu));
 }
 
@@ -67,8 +69,14 @@ double pinvgauss(double q, double mu, double phi, int lower_tail, int log_p)
     if (ISNAN(q) || ISNAN(mu) || ISNAN(phi))
 	return q + mu + phi;
 #endif
-    if (mu <= 0.0 || phi <= 0.0)
+    if (mu <= 0.0)
         return R_NaN;
+    if (phi <= 0)
+    {
+	if (phi < 0) return R_NaN;
+	/* phi == 0 : */
+	return (q == 0) ? ACT_DT_0 : ACT_DT_1;
+    }
 
     if (q < 0)
         return ACT_DT_0;
@@ -341,83 +349,3 @@ double mgfinvgauss(double t, double mu, double phi, int give_log)
 
     return ACT_D_exp((1 - sqrt(1 - 2 * phi * t))/phi);
 }
-
-
-/*
- * DEPRECATED VERSIONS
- */
-double minvGauss(double order, double nu, double lambda, int give_log)
-{
-#ifdef IEEE_754
-    if (ISNAN(order) || ISNAN(nu) || ISNAN(lambda))
-	return order + nu + lambda;
-#endif
-    if (!R_FINITE(nu) ||
-        !R_FINITE(lambda) ||
-        !R_FINITE(order) ||
-        nu <= 0.0 ||
-        lambda <= 0.0 ||
-        (int) order != order)
-        return R_NaN;
-
-    /* Trivial case */
-    if (order == 0.0)
-        return 0.0;
-
-    int i, n = order;
-    double z = 0.0;
-
-    for (i = 0; i < n; i++)
-        z += R_pow_di(nu, n) * gammafn(n + i) *
-            R_pow_di(2.0 * lambda/nu, -i) /
-            (gammafn(i + 1) * gammafn(n - i));
-    return z;
-}
-
-double levinvGauss(double limit, double nu, double lambda, double order,
-                   int give_log)
-{
-#ifdef IEEE_754
-    if (ISNAN(limit) || ISNAN(nu) || ISNAN(lambda) || ISNAN(order))
-	return limit + nu + lambda + order;
-#endif
-    if (!R_FINITE(nu)     ||
-        !R_FINITE(lambda) ||
-        !R_FINITE(order)  ||
-        nu <= 0.0    ||
-        lambda < 0.0 ||
-        order != 1.0)
-        return R_NaN;
-
-    if (limit <= 0.0)
-        return 0.0;
-
-    /* From R, order == 1 */
-    double tmp, y, z;
-
-    tmp = sqrt(lambda/limit);
-    y = (limit + nu)/nu;
-    z = (limit - nu)/nu;
-
-    return limit - nu * z * pnorm(z * tmp, 0.0, 1.0, 1, 0)
-        - nu * y * exp(2.0 * lambda/nu) * pnorm(-y * tmp, 0.0, 1.0, 1, 0);
-}
-
-double mgfinvGauss(double x, double nu, double lambda, int give_log)
-{
-#ifdef IEEE_754
-    if (ISNAN(x) || ISNAN(nu) || ISNAN(lambda))
-	return x + nu + lambda;
-#endif
-    if (!R_FINITE(nu) ||
-        !R_FINITE(lambda) ||
-        nu <= 0.0 ||
-        lambda < 0.0 ||
-        x > lambda/(2.0 * nu * nu))
-        return R_NaN;
-
-    if (x == 0.0)
-        return ACT_D_exp(0.0);
-
-    return ACT_D_exp(lambda / nu * (1.0 - sqrt(1.0 - 2.0 * nu * nu * x/lambda)));
-}
