@@ -4,6 +4,12 @@
  *  functions, and to simulate random variates for the logarithmic
  *  discrete distribution. See ../R/Logarithmic.R for details.
  *
+ *  We work with the probability mass function expressed as
+ *
+ *    a * p^x / x,  x = 1, 2, ...
+ *
+ *  with a = -1/log(1 - p).
+ *
  *  AUTHOR: Vincent Goulet <vincent.goulet@act.ulaval.ca>
  */
 
@@ -14,13 +20,6 @@
 
 double dlogarithmic(double x, double p, int give_log)
 {
-    /*  We work with the probability mass function expressed as
-     *
-     *  a * p^x / x,  x = 1, 2, ...
-     *
-     *  with a = -1/log(1 - p).
-     */
-
 #ifdef IEEE_754
     if (ISNAN(x) || ISNAN(p))
 	return x + p;
@@ -220,7 +219,7 @@ double rlogarithmic(double p)
 	    s *= p * (x - 1.0)/x;
 	}
 
-	return(x);
+	return x;
     }
 
     /* else (p >= 0.95) */
@@ -233,8 +232,8 @@ double rlogarithmic(double p)
 	double u = unif_rand();
 	double q = -expm1(r * u);
 
-	if (v <= (q * q)) return(round(1.0 + log(v)/log(q)));
-	if (v <= q)       return(1.0); /* case q^2 < v <= q */
-	return(2.0);		       /* case v > q */
+	if (v <= (q * q)) return floor(1.0 + log(v)/log(q));
+	if (v <= q)       return 2.0; /* case q^2 < v <= q */
+	return 1.0;		       /* case v > q */
     }
 }
